extern crate gtk;
use gtk::prelude::*;
use gtk::{Button, Window, WindowType};
use std::cell::RefCell;
use std::rc::Rc;

enum Operators {
    PLUS,
    MINUS,
    DIVIDE,
    MULTIPLY,
}

struct Calc {
    left: String,
    right: String,
    operator: Option<Operators>,
    label: gtk::Label,
}

impl Calc {
    fn handle_number(&mut self, button: String) {
        match self.operator {
            None => self.left.push_str(&button),
            Some(_) => self.right.push_str(&button),
        };
        self.display();
    }
    fn eval(&mut self) {
        let left = self.left.parse::<i32>().unwrap_or(0);
        let right = self.right.parse::<i32>().unwrap_or(0);
        let ans = match self.operator {
            Some(Operators::PLUS) => left + right,
            Some(Operators::MINUS) => left - right,
            Some(Operators::DIVIDE) => left / right,
            Some(Operators::MULTIPLY) => left * right,
            None => panic!("oh noes"),
        };
        self.right = String::new();
        self.operator = None;
        self.left = ans.to_string();
        self.display();
    }
    fn new() -> Self {
        Calc {
            left: String::new(),
            right: String::new(),
            operator: None,
            label: gtk::Label::new(Some("")),
        }
    }
    fn display(&self) {
        let text = match self.operator {
            Some(Operators::PLUS) => self.left.clone() + "+" + &self.right,
            Some(Operators::MINUS) => self.left.clone() + "-" + &self.right,
            Some(Operators::DIVIDE) => self.left.clone() + "/" + &self.right,
            Some(Operators::MULTIPLY) => self.left.clone() + "*" + &self.right,
            None => self.left.clone(),
        };
        self.label.set_label(&text);
    }
}

fn main() {
    if gtk::init().is_err() {
        println!("failed to init GTK");
        return;
    }
    let calc = Rc::new(RefCell::new(Calc::new()));
    let window: Window = Window::new(WindowType::Toplevel);
    window.set_title("Rust calc");
    let container = gtk::Box::new(gtk::Orientation::Vertical, 1);
    // row 0
    let cal0 = calc.clone();
    container.add(&cal0.borrow().label);

    let keys: [[&'static str; 3]; 5] = [
        ["1", "2", "3"],
        ["4", "5", "6"],
        ["7", "8", "9"],
        ["0", "-", "+"],
        ["*", "/", "="],
    ];
    for row in &keys {
        let gui_row = gtk::Box::new(gtk::Orientation::Horizontal, 1);
        for button in row {
            let gui_button = Button::new_with_label(button);
            gui_row.add(&gui_button);
            let cal_copy = calc.clone();
            gui_button.connect_clicked(move |but| {
                let label = but.get_label().unwrap();
                match label.chars().next().unwrap() {
                    '0'...'9' => cal_copy.borrow_mut().handle_number(label),
                    '-' | '+' | '*' | '/' => {
                        let mut cal_ops_borrowed = cal_copy.borrow_mut();
                        cal_ops_borrowed.operator = Some(match label.as_ref() {
                            "-" => Operators::MINUS,
                            "+" => Operators::PLUS,
                            "*" => Operators::MULTIPLY,
                            "/" => Operators::DIVIDE,
                            _ => panic!("oh noes, unknown operator"),
                        });
                        cal_ops_borrowed.display();
                    }
                    '=' => cal_copy.borrow_mut().eval(),
                    _ => panic!("oh noes unknowen button label"),
                };
            });
        }
        container.add(&gui_row);
    }

    window.add(&container);
    window.show_all();
    window.connect_delete_event(|_, _| {
        gtk::main_quit();
        Inhibit(false)
    });
    gtk::main();
}
